/*
 * sample_main.c: Main routines for the sample plugin.
 *
 * Copyright(c) 2002 Cerulean Studios
 *
 */

#include "sample.h"
#include "resource.h"

char			*home_directory = NULL;

HINSTANCE		hInstance;

/* Happy globals to manage stuff.
 */

HBITMAP			listentry_bmp = NULL;

int				parentID = -1;
int				sectionID = -1;
int				uniqueID = -1;
int				aliasID = -1;
int				keyboard_alias_id = -1;
int				broadcast_id = -1;
	
plugin_function_send plugin_send = NULL; 

/******************************************************************************
 p l u g i n _ v e r s i o n ()
******************************************************************************

  Currently supporting version 1 of plugins.
*/
 
int __declspec(dllexport)plugin_version(void)
{
	return 1;
}

/******************************************************************************
 p l u g i n _ m a i n ()
******************************************************************************

  plugin_main() gets called whenever the UI has something to tell us.  This includes
  telling the plugin when to start, stop, unload, show preferences, etc.
*/
 
int __declspec(dllexport)plugin_main(char *event, void *data) 
{   
	if (!_stricmp(event,"load")) { 
	   struct plugin_info_t *pi = (struct plugin_info_t *)data;

	   if (pi) {

			/* Fill in plugin information.
			 */

			strcpy(pi->guid, MYGUID); 
			strcpy(pi->name, "Sample Plugin"); 
			strcpy(pi->company, "Cerulean Studios"); 
			strcpy(pi->version, "1.0"); 
			strcpy(pi->description, "This is a sample plugin.");
         
			/* Extract what we need and store locally.
			 */

			home_directory = (char *)malloc(strlen(pi->config_directory)+1);
			strcpy(home_directory, pi->config_directory);

			plugin_send = pi->plugin_send; 
	   } 
	} else if (!_stricmp(event, "initialLoad")) {

		/* First time a plugin is loaded - can do install wizardish stuff here.
		 */

	} else if (!_stricmp(event, "start")) {
		
		struct plugin_prefs_t		pp;
        struct list_entry_t			ple; 
		struct alias_t				ea;
		struct keyboard_alias_t		ka; 
		struct connection_enum_t	cet;
		struct contactlist_enum_t	cet2;
		struct message_t			mst;
		struct message_broadcast_t	mbt;

		struct menu_info_t			*menu_info;

		trillianInitialize(pp);

		/* Note the different function.
		 */

		trillianListInitialize(ple);

		trillianInitialize(ea);
		trillianInitialize(ka);
		trillianInitialize(cet);
		trillianInitialize(cet2);
		trillianInitialize(mst);
		trillianInitialize(mbt);
		
		/* Now's a good time to read a configuration file.
		 */

		sample_util_readConfig();

		if (listentry_bmp == NULL)
			listentry_bmp = LoadBitmap(hInstance, MAKEINTRESOURCE(IDB_THUNDERSTORMS));

		/* Initialize preferences.
		 */

		pp.enabled						= 1; 
		pp.pref_name					= "SAMPLE"; 
		pp.sub_entry					= NULL; 

		plugin_send(MYGUID, "prefsInitialize", &pp); 

		/* Initialize aliases (such as /slap in an edit box).
		 */

		if (aliasID == -1) {
			ea.alias_id = 0;
			ea.text = "/slap";
			ea.callback = aliasCallback;
			ea.data = NULL;

			aliasID = plugin_send(MYGUID, "editAliasRegister", &ea);
		}

		/* Initialize keyboard aliases (such as ctrl-s)
		 */

		if (keyboard_alias_id == -1) {
			ka.ctrl = 1; 
			ka.alias_id = -1; 
			ka.key = 's'; 
			ka.callback = aliasCallback; 
			ka.data = NULL; 

			keyboard_alias_id = plugin_send(MYGUID, "keyboardAliasRegister", (void *)&ka); 
		}

		/* Populate contact list.
		 */

		/* -1 means use default.
		 */

        ple.previous_id					= -2;	/* This controls placement of new entry, before (-2) or last (-1) */
        
		ple.section						= 1;	/* This is a section */
		ple.drag_and_drop				= 1;	/* Allow drag and drop, though it won't matter for now */
        ple.expanded					= 1;	

		/* Add the main section.
		 */

		ple.tooltip						= "Sample section.";
		ple.text						= "My Section";

		/* 
		 * Real names allow multiple plugins to share a single section - Trillian's weather,
		 * news, and stocks plugin use the "web" section, for example.
		 */

		ple.real_name					= "sample";
		plugin_send(MYGUID, "listAddEntry", (void *)&ple); 
			
		/* 
		 * Set our global sectionID for later.  Note that we get the sectionID from the structure
		 * after it's been populated.
		 */

		sectionID = ple.section_id;

		/* Add a group.  Flip section/group around.  Group ID is the RETURN value, not in struct.
		 */

		ple.group						= 1;
		ple.section						= 0;
		ple.tooltip						= "Sample";
		ple.text						= "Sample"; 
		parentID = plugin_send(MYGUID, "listAddEntry", (void *)&ple);

		/* 
		 * Add a sample list entry with an icon.  Setup a callback for it so we can monitor
		 * when someone clicks the entry.
		 */

		ple.previous_id					= -1; 
		ple.section_id					= sectionID;
		ple.parent_id					= parentID;
		ple.unique_id					= -1; 
		ple.group						= 0;
		ple.section						= 0; 
		ple.inline_editing				= 0;

		ple.drag_and_drop				= 1; 
		ple.expanded					= 1; 

		ple.tooltip						= "stormy sample";
		ple.text						= "stormy sample";
		
		/* 
		 * Callbacks can take a data parameter of whatever you want.  It's best to pass a pointer
		 * to data stored elsewhere, as the API provides no mechanism to free this memory.
		 */

		ple.callback					= listCallback;
		ple.data						= (void *)"http://www.ceruleanstudios.com";

		/* Setup an icon for this entry.  
		 */

		ple.left_icons = (struct list_bmp_t *)malloc(sizeof(struct list_bmp_t));
		trillianInitialize((*ple.left_icons));
		
		ple.left_icons->num_states = 1;

		ple.left_icons->bitmap = (HBITMAP *)malloc(sizeof(HBITMAP));
		ple.left_icons->bitmap[0] = listentry_bmp;

		ple.left_icons->location = (RECT *)malloc(sizeof(RECT));
		ple.left_icons->location[0].left = 0;
		ple.left_icons->location[0].top = 0;
		ple.left_icons->location[0].bottom = 16;
		ple.left_icons->location[0].right = 16;

		ple.num_left_icons = 1;

		/* 
		 * Setup a menu.  There are a few ways to do this, but in this example we're telling Trillian
		 * to use the 'callback' method of menu creation.  This way, we are notified when someone
		 * right clicks an entry and can create menus on-the-fly.  
		 */

		ple.menu_entry = (struct menu_entry_t *)malloc(sizeof(struct menu_entry_t));
		trillianInitialize((*ple.menu_entry));

		ple.menu_entry->type = MENU_CALLBACK;
		ple.menu_entry->callback = menuCallback;

		/* 
		 * Menu item gets it's own info field, which can be whatever you want.  Here, a structure
		 * is defined with some hopefully useful data.
		 */

		menu_info = (struct menu_info_t *)malloc(sizeof(struct menu_info_t));
		memset(menu_info, 0, sizeof(struct menu_info_t));

		menu_info->menu_type = MENU_TYPE_SAMPLE;
		menu_info->url = _strdup("http://www.ceruleanstudios.com");

		ple.menu_entry->data = menu_info;

		/* You'll need the UniqueID for any further manipulation, so store it somewhere nice.
		 */

		uniqueID = plugin_send(MYGUID, "listAddEntry", (void *)&ple); 

		/*
		 * Free what needs to be freed.  Other things (menu info in particular) are freed
		 * after they've been used.
		 */

		free(ple.menu_entry);
		free(ple.left_icons->bitmap);
		free(ple.left_icons->location);
		free(ple.left_icons);

		/* 
		 * Ask Trillian to enumerate the connection list and contact list.  Note that connections need to be created before
		 * Trillian is aware of them - i.e stuff in your connection manager that's never been connected won't be detected.
		 *
		 * Offline contacts ARE sent, however.
		 */

		cet.callback = connectCallback;
		cet.medium = "all";					// Can also be "AIM", "ICQ", etc.
		
		plugin_send(MYGUID, "connectionEnumerate", &cet);

		cet2.connection_id = -1;			// No specific CID requested.
		cet2.medium = "all";
		cet2.callback = contactCallback;

		plugin_send(MYGUID, "contactlistEnumerate", &cet2);

		/* 
		 * Sending a message - if you wish to send a message change the values below and uncomment the plugin_send and
		 * it should work fine. 
		 */

		mst.medium = "AIM";
		mst.connection_id = 0;
		mst.name = "MyScreenNameOnAOLThisWon'tReallyWork";
		mst.type = "outgoing";
		mst.text = "This is a plugin controlling your world.";

		/*plugin_send(MYGUID, "messageSend", &mst);*/

		/* 
		 * Tell Trillian we want to receive inbound instant messages, i.e "enable us for broadcast transmissions".  All we
		 * need to give is a callback.
		 *
		 * NOTE: broadcast_id will be returned from plugin_send - make sure to close the broadcast transmission when the
		 * plugin is unloaded for any reason.  To do so, fill out a message_broadcast_t structure with broadcast_id 
		 * set properly and message:
		 *	
		 * plugin_send(MYGUID, "messageBroadcastDisable", &mbt);
		 */

		if (broadcast_id == -1) {
			mbt.callback = broadCallback;
			broadcast_id = plugin_send(MYGUID, "messageBroadcastEnable", &mbt);
		}

	} else if (!_stricmp(event,"stop") && plugin_send) {
		struct list_entry_t			ple;
		struct alias_t				ea;
		struct keyboard_alias_t		ka; 
		struct message_broadcast_t	mbt;
		
		trillianListInitialize(ple);
		trillianInitialize(ea);
		trillianInitialize(ka);
		trillianInitialize(mbt);

		/* Write our configuration.
		 */

		sample_util_writeConfig();

		/* Destroy the sample group.
		 */

		ple.font.hover_background		= -1;
		ple.font.hover_foreground		= -1; 
		ple.font.select_foreground		= -1; 
		ple.font.select_background		= -1; 
		ple.font.foreground				= -1; 
		ple.font.background				= -1; 

		ple.previous_id					= -1; 
		ple.section_id					= -1;
		ple.parent_id					= parentID;
		ple.unique_id					= parentID; 
		ple.group						= 0;
		ple.section						= 0; 
		ple.inline_editing				= 0;

		ple.drag_and_drop				= 1; 
		ple.expanded					= 1; 

		plugin_send(MYGUID, "listRemoveEntry", (void *)&ple); 

		/* If "My Section" is unused, destroy it as well.
		 */

		ple.section_id = sectionID;
		if (plugin_send(MYGUID, "listGetSectionCount", (void *)&ple) == 0) {
			ple.previous_id				= -1; 
			ple.section_id				= sectionID;
			ple.parent_id				= -1;
			ple.unique_id				= -1; 

			plugin_send(MYGUID, "listRemoveEntry", (void *)&ple); 

			sectionID = -1;
		}

		/* Remove alias.
		 */

		ea.alias_id = aliasID;

		plugin_send(MYGUID, "editAliasUnregister", &ea);
		aliasID = -1;

		/* Remove keyboard alias.
		 */
            
        ka.alias_id = keyboard_alias_id; 
        plugin_send(MYGUID, "keyboardAliasUnregister", (void *)&ka); 
		keyboard_alias_id = -1;

		/* Turn off broadcasting.
		 */

		mbt.broadcast_id = broadcast_id;
		plugin_send(MYGUID, "messageBroadcastDisable", &mbt);
		broadcast_id = -1;

	} else if (!_stricmp(event, "unload")) {			

		/* Clean up. 
		 */

		if (home_directory) {
			free(home_directory);
			home_directory = NULL;
		}

		if (listentry_bmp) {
			DeleteObject(listentry_bmp);
			listentry_bmp = NULL;
		}

		if (prefs_window != NULL) { 
			myDestroyWindow(prefs_window); 
			prefs_window = NULL; 
		} 

		plugin_send = NULL;

		return 0;
	} else if (!_stricmp(event, "finalUnload")) {

		/* 
		 * Final unload means the plugin is actually being removed.  Plugins should clean up
		 * saved settings here (delete INI files, etc).
		 */

	} else if (!_stricmp(event, "prefsShow")) {
		struct plugin_prefs_show_t	*pps = (struct plugin_prefs_show_t *)data;

		/* 
		 * Trillian is telling us to show our preferences dialog - do so only if we find our
		 * name.
		 */

		if (!strcmp(pps->pref_name, "SAMPLE")) {
			int				prefDialog();

			strcpy(pps->prefs_info.name, "Sample Plugin");
			strcpy(pps->prefs_info.description, "Making plugins out of nothing at all.");

			/* This can be an HBITMAP if you've got a pretty icon for your preferences screen.
			 */

			pps->prefs_info.bitmap = 0;

			/* Show or hide?
			 */

			if (pps->show == 1) {

				if (prefs_window == NULL) {
					prefs_window = myCreateDialog(hInstance, MAKEINTRESOURCE(IDD_PREFS_DIALOG), pps->hwnd, (DLGPROC)prefDialog);
					SetWindowPos(prefs_window, 0, pps->x, pps->y, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
				}

				ShowWindow(prefs_window, SW_SHOW);
			} else if (pps->show == 0) {
				if (prefs_window != NULL)
					ShowWindow(prefs_window, SW_HIDE);
			}
		}
	} else if (!_stricmp(event, "prefsAction")) {
		struct plugin_prefs_action_t	*ppa = (struct plugin_prefs_action_t *)data;

		/* Take action:
		 *
		 * 0 - Cancel
		 * 1 - Apply
		 * 2 - OK
		 */

		if (ppa->type == 0) {
			if (prefs_window != NULL) {
				myDestroyWindow(prefs_window);
				prefs_window = NULL;
			}
		} else if (ppa->type == 1) {
			if (prefs_window != NULL) 
				prefResult(TRUE);

			sample_util_writeConfig();
		} else if (ppa->type == 2) {
			if (prefs_window != NULL) {
				prefResult(TRUE);
				myDestroyWindow(prefs_window);
				prefs_window = NULL;
			}

			sample_util_writeConfig();
		}		
	}

	return 0; 
} 

/******************************************************************************
 D l l M a i n ()
******************************************************************************/

int WINAPI DllMain(HANDLE hinstDLL, unsigned long dwReason, void *reserved)
{
	hInstance = (HINSTANCE)hinstDLL;

	switch (dwReason) {

		case DLL_THREAD_DETACH:
			break;

        case DLL_PROCESS_DETACH:
			break;
	}

	return 1;
}

