/*
 * sample_util.c: Utility routines.
 *	 
 * Copyright(c) 2002 Cerulean Studios
 *
 */

#include "sample.h"
#include "resource.h"

/******************************************************************************
 l i s t C a l l b a c k ()
******************************************************************************/

int listCallback(int windowID, char *subwindow, char *event, void *data, void *userData) 
{
	if (!strcmp(event, "list_leftDoubleClick")) {

		/* Open a browser with our user data.
		 */

		ShellExecute(mMainWnd, "open", (char *)userData, 0, 0, SW_SHOW);
		
	}	

	return 0;
}

/******************************************************************************
 t o o l t i p C a l l b a c k ()
******************************************************************************/

int tooltipCallback(int windowID, char *subwindow, char *event, void *data, void *userData) 
{
	if (!strcmp(event, "alert_linkClick")) {
		if (!strcmp(data, "web")) {

			/* User data contains whatever we want.  NOTE that We don't free it here, though!
			 */

		}
	} else if (!strcmp(event, "alert_destroy")) {

		/* Clean up user data
		 */

		if (userData != NULL)
			free(userData);
	}

	return 0;
}

/******************************************************************************
 a l i a s C a l l b a c k ()
******************************************************************************/

int aliasCallback(int windowID, char *subwindow, char *event, void *data, void *userData) 
{
	if (!_stricmp(event, "edit_aliasRequest")) { 
		struct alias_request_t	*alias_t = (struct alias_request_t *)data;

		if (!strcmp(alias_t->alias, "/slap")) {
			alias_t->text = _strdup("/me slaps it's own face!");
			return 0;
		}
	} else if (!_stricmp(event, "edit_aliasFree")) {
		if (data)
			free(data);
	} else if (!_stricmp(event,"keyboard_aliasRequest")) { 
		struct keyboard_alias_request_t *kar = (struct keyboard_alias_request_t *)data; 
            
		if (kar->key == 's') {
			kar->text = _strdup("/me slaps it's own face!");
			return 0;
		}
	} else if (!_stricmp(event,"keyboard_aliasFree")) { 
		char	*c = (char *)data; 

		if (c)
			free(c);
	}
	
	return 1;
}

/******************************************************************************
 c o n n e c t C a l l b a c k ()
******************************************************************************/

int connectCallback(int windowID, char *subwindow, char *event, void *data, void *userData) 
{
	struct connection_entry_t	*entry = (struct connection_entry_t *)data;

	/* Entry pointer holds the data we need.
	 */

	if (!strcmp(event, "enum_start")) {

		/* List beginning, no data is sent.
		 */

	} else if (!strcmp(event, "enum_end")) {

		/* 
		 * Finished spooling us the list after a request; we now know all available connections, so we can populate
		 * a list of sorts here.
		 */

	} else if (!strcmp(event, "enum_add")) {
		char	buffer[1024];

		/* 
		 * Getting a new connection entry.
		 *
		 *	struct connection_entry_t {
		 *		unsigned int		struct_size; 
		 * 
		 *		char				*medium; 
		 *		int                 connection_id; 
		 *
		 *		char				*name; 
		 *		char				*status; 
		 *	};
		 *
		 * Medium can be "AIM", "ICQ", "MSN", "YAHOO", "IRC".
		 *
		 * Connection ID is Trillian's internal ID for this connection; you'll need this stored somewhere.
		 *
		 * Name is the screenname/hotmail passport/whatever.
		 *
		 * Status is one of "Online" / "Offline" / "Away" / etc... can basically be anything Trillian can be.
		 */

		sprintf(buffer, "Connection - %s(%s) on %s", entry->name, entry->status, entry->medium);
		MessageBox(HWND_DESKTOP, buffer, "Enumerated Connection", MB_OK);
	}

	return 0;
}

/******************************************************************************
 c o n t a c t C a l l b a c k ()
******************************************************************************/

int contactCallback(int windowID, char *subwindow, char *event, void *data, void *userData) 
{
	struct contactlist_entry_t	*entry = (struct contactlist_entry_t *)data;

	/* Same structure as connectCallback.
	 */

	if (!strcmp(event, "enum_start")) {
		;
	} else if (!strcmp(event, "enum_end")) {
		;
	} else if (!strcmp(event, "enum_add")) {
		char	buffer[1024];

		/* 
		 * See contactlist_entry_t - again, the same data pretty much.  Status can be one of any number of weird
		 * statuses that each medium supports... it might be easiest to say:
		 * 
		 *	if (!_stricmp(entry->status, "online") || !_stricmp(entry->status, "recnet-offline")) 
		 *		user is online
		 *	
		 *	if (!_stricmp(entry->status, "offline") || !_stricmp(entry->status, "recent-offline")) 
		 *		user is offline
		 *	
		 *	else 
		 *		user is away
		 *
		 */

		sprintf(buffer, "Contact - %s on %s", entry->name, entry->medium);
		MessageBox(HWND_DESKTOP, buffer, "Enumerated Contact", MB_OK);
	}

	return 0;
}

/******************************************************************************
 b r o a d C a l l b a c k ()
******************************************************************************/

int broadCallback(int windowID, char *subwindow, char *event, void *data, void *userData) 
{
	struct message_t	*msg = (struct message_t *)data;
  
	/*
	 *	struct message_t {
	 *		unsigned int		struct_size; 
	 *
	 *		char				*medium; 
	 *		int                 connection_id; 
	 *
	 *		char				*name; 
	 *		char				*type; 
	 *		char				*text; 
	 *		char				*location;
	 *
	 *		void				*extra_information; 
	 *	}; 
	 *
	 * 
	 * Note that text is HTML formatted, as this is the internal standard for Trillian to keep things sane across
	 * multiple mediums.  Anything you receive should already be converted into HTML for you.
	 *
	 */

	if (!strcmp(event, "message_broadcast")) {
		char	buffer[1024];

		sprintf(buffer, "%s - %s", msg->name, msg->text);
		MessageBox(HWND_DESKTOP, buffer, msg->type, MB_OK);
	}

	return 0;
}

/******************************************************************************
 s a m p l e  _ u t i l _ t o o l t i p ()
******************************************************************************/

void sample_util_tooltip(char *type, char *message, char *data)
{
	struct alert_t tooltip;
	
	/* 
	 * Does a systray alert.  The type comes back to us later, and also controls the icon.  Skins
	 * can define icons of 'type' and Trillian will use them.
	 */

	trillianInitialize(tooltip);
	
	tooltip.text = message;
	tooltip.type = type;
	tooltip.link = 1;
	tooltip.callback = tooltipCallback;
	tooltip.data = data;

	plugin_send(MYGUID, "systraySetAlert", &tooltip); 

	return;
}

/******************************************************************************
 s a m p l e  _ u t i l _ w r i t e C o n f i g ()
******************************************************************************/

void sample_util_writeConfig(void)
{
	return;
}

/******************************************************************************
 s a m p l e  _ u t i l _ r e a d C o n f i g ()
******************************************************************************/

void sample_util_readConfig(void)
{
	return;
}
