/*
 * sample_windows.c: Windows routines
 *	 
 * Copyright(c) 2002 Cerulean Studios
 *
 */

#include "sample.h"
#include "resource.h"

HWND		prefs_window = NULL;

/******************************************************************************
 p r e f D i a l o g ()
******************************************************************************/

int prefDialog(HWND hWnd, UINT message, WPARAM wparam, LPARAM lparam)
{
	switch (message) {

		case WM_INITDIALOG:
			break;

		case WM_COMMAND:

			switch ((int)LOWORD(wparam)) {

				default:
					break;

			}

			break;
	}

	return 0;
}

/******************************************************************************
 p r e f R e s u l t ()
******************************************************************************/

int prefResult(int value)
{
	if (value == TRUE) {

	}

	return 0;
}

/******************************************************************************
 d e s t r o y M e n u ()
******************************************************************************/

void destroyMenu(struct menu_entry_t *menu)
{
	while (menu != NULL) {
		struct menu_entry_t	*next = menu->next_menu;
		struct menu_info_t	*info = menu->data;

		destroyMenu(menu->sub_menu);

		if (menu->icon)
			free(menu->icon);
		if (menu->text)
			free(menu->text);
		if (menu->shortcut)
			free(menu->shortcut);
		free(menu);

		menu = next;
	}
} 

/******************************************************************************
 c r e a t e M e n u I t e m ()
******************************************************************************/

struct menu_entry_t *createMenuItem(void)
{
	struct menu_entry_t	*menu = (struct menu_entry_t*)malloc(sizeof(struct menu_entry_t));
	
	memset(menu, 0, sizeof(struct menu_entry_t));
	menu->menu_id = -1;
	menu->type = MENU_TEXT;

	return menu;
}

/******************************************************************************
 a d d M e n u I t e m ()
******************************************************************************/

void addMenuItem(struct menu_entry_t **main_menu, struct menu_entry_t *mi)
{
	if (*main_menu) {
		struct menu_entry_t	*current = *main_menu;

		while (current) {
			if (!current->next_menu) {
				current->next_menu = mi;
				break;
			} else
				current = current->next_menu;
		}
	} else
		*main_menu = mi;
}

/******************************************************************************
 c l o n e I n f o I t e m ()
******************************************************************************/

struct menu_info_t *cloneInfoItem(struct menu_info_t *info)
{
	struct menu_info_t	*sub_info = (struct menu_info_t *)malloc(sizeof(struct menu_info_t));
	
	memset(sub_info, 0, sizeof(struct menu_info_t));

	sub_info->menu_type = info->menu_type;
	sub_info->menu_id = 0;
	if (info->url)
		sub_info->url = _strdup(info->url);
	else
		sub_info->url = NULL;

	return sub_info;
}

/******************************************************************************
 m e n u C a l l b a c k ()
******************************************************************************/

int menuCallback(int windowID, char *subwindow, char *event, void *data, void *userData) 
{	
	if (!strcmp(event, "menu-open")) {
		struct menu_entry_t	*menu = NULL;
		struct menu_entry_t	*mi;
		struct menu_entry_t	*orig_item;

		struct menu_info_t	*info;

		/* Grab information pointer and dynamically create a menu based on given data.
		 */

		orig_item = (struct menu_entry_t *)data;
		info = orig_item->data;
		
		switch (info->menu_type) {
		
			case MENU_TYPE_SAMPLE:
				{
				struct menu_info_t		*sub_info;

				mi = createMenuItem();
				mi->text = (char *)malloc(strlen("Click me for fun")+1);
				strcpy(mi->text, "Click me for fun");
				mi->callback = menuCallback;

				/* Skins define icons.  Check is global.  
				 */

				mi->icon = _strdup("check");

				sub_info = cloneInfoItem(info);
				sub_info->menu_id = IDM_SAMPLE;
				mi->data = sub_info;
				addMenuItem(&menu, mi);

				}
				break;

			default:
				break;
		}

		return (int)menu;
	} else if (!strcmp(event, "menu-select")) {
		struct menu_entry_t		*orig_item;
		struct menu_entry_t		*info;

		/* Menu entry has been clicked.  Grab our info pointers and figure out what's going on.
		 */

		orig_item = (struct menu_entry_t *)data;
		info = orig_item->data;

		switch (info->menu_id) {

			case IDM_SAMPLE:

				/* 
				 * Allocated memory from _strdup gets passed around until the tooltip expires,
				 * at which point it's freed.  See the tooltip callback.
				 */

				sample_util_tooltip("sample", "Sample Plugin: Hello there!", _strdup("http://www.ceruleanstudios.com"));
				break;

			default:
				break;
		}

	} else if (!strcmp(event, "menu-open-finish")) {
		struct menu_entry_t	*menu;

		/* Only destroy the menu structs here.
		 */

		menu = (struct menu_entry_t *)data;
		destroyMenu(menu);
	} else if (!strcmp(event, "menu-destroy")) {
		struct menu_info_t	*info;

		/* Destroy our custom data here.
		 */

		info = (struct menu_info_t *)userData;

		if (info) {
			if (info->url) 
				free(info->url);
			free(info);
		}
	}

	return 0;
}

/******************************************************************************
 m y C r e a t e D i a l o g ()
******************************************************************************/

HWND myCreateDialog(HINSTANCE hInstance, LPCTSTR templateStr, HWND parent, DLGPROC dlg)
{
	struct dialog_entry_t		det;

	trillianInitialize(det);
						
	det.hwnd = CreateDialog(hInstance, templateStr, parent, dlg);
	
	plugin_send(MYGUID, "dialogAdd", &det);   

	return det.hwnd;
}
		
/******************************************************************************
 m y D e s t r o y W i n d o w ()
******************************************************************************/

void myDestroyWindow(HWND hWnd)
{
	struct dialog_entry_t		det;
	
	trillianInitialize(det);
	det.hwnd = hWnd;

	plugin_send(MYGUID, "dialogRemove", &det);

	DestroyWindow(hWnd);
}
